﻿using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Encodings.IntegerVectorEncoding;
using HeuristicLab.Encodings.RealVectorEncoding;
using HeuristicLab.Operators;
using HeuristicLab.Optimization;
using HeuristicLab.Parameters;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Problems.MetaOptimization.Encodings.Crossovers {
  /// <summary>
  /// 
  /// </summary>
  [Item("ParameterConfigurationCrossover", "TODO")]
  [StorableClass]
  public class ParameterConfigurationCrossover : SingleSuccessorOperator, IParameterConfigurationOperator, ICrossover {
    public override bool CanChangeName {
      get { return false; }
    }

    public ILookupParameter<IRandom> RandomParameter {
      get { return (LookupParameter<IRandom>)Parameters["Random"]; }
    }
    public ILookupParameter<ItemArray<IValueConfiguration>> ParentsParameter {
      get { return (ScopeTreeLookupParameter<IValueConfiguration>)Parameters["Parents"]; }
    }
    public ILookupParameter<IValueConfiguration> ChildParameter {
      get { return (ILookupParameter<IValueConfiguration>)Parameters["Child"]; }
    }

    private IIntegerVectorCrossover integerVectorCrossover;
    public IIntegerVectorCrossover IntegerVectorCrossover {
      get { return integerVectorCrossover; }
      set { integerVectorCrossover = value; }
    }
    private IRealVectorCrossover realVectorCrossover;
    public IRealVectorCrossover RealVectorCrossover {
      get { return realVectorCrossover; }
      set { realVectorCrossover = value; }
    }

    [StorableConstructor]
    protected ParameterConfigurationCrossover(bool deserializing) : base(deserializing) { }
    protected ParameterConfigurationCrossover(ParameterConfigurationCrossover original, Cloner cloner) : base(original, cloner) { }
    public ParameterConfigurationCrossover()
      : base() {
      Parameters.Add(new LookupParameter<IRandom>("Random", "The pseudo random number generator which should be used for stochastic crossover operators."));
      Parameters.Add(new ScopeTreeLookupParameter<IValueConfiguration>("Parents", "The parent vectors which should be crossed."));
      ParentsParameter.ActualName = "ParameterConfigurationTree";
      Parameters.Add(new LookupParameter<IValueConfiguration>("Child", "The child vector resulting from the crossover."));
      ChildParameter.ActualName = "ParameterConfigurationTree";
    }
    public override IDeepCloneable Clone(Cloner cloner) {
      return new ParameterConfigurationCrossover(this, cloner);
    }

    public override IOperation Apply() {
      IValueConfiguration child1 = (IValueConfiguration)ParentsParameter.ActualValue[0].Clone();
      IValueConfiguration child2 = (IValueConfiguration)ParentsParameter.ActualValue[1];

      child1.Cross(child2, RandomParameter.ActualValue);
      this.ChildParameter.ActualValue = child1;
      return base.Apply();
    }

    private IntValue CrossInteger(IParameterConfiguration parameter1, IParameterConfiguration parameter2) {
      IntegerVector integerChild = HeuristicLab.Encodings.IntegerVectorEncoding.DiscreteCrossover.Apply(
        RandomParameter.ActualValue,
        new IntegerVector(new IntArray(new int[] { ((IntValue)parameter1.ActualValue.Value).Value })),
        new IntegerVector(new IntArray(new int[] { ((IntValue)parameter2.ActualValue.Value).Value })));
      return new IntValue(integerChild[0]);
    }

  }
}
